<?php

namespace Drupal\queue_order_ui\Form;

use Drupal\Core\Form\ConfigFormBase;
use Drupal\Core\Form\FormStateInterface;

/**
 * Class Order.
 *
 * @package Drupal\queue_order_ui\Form
 */
class Order extends ConfigFormBase {

  const CONFIG = 'queue_order.settings';

  /**
   * {@inheritdoc}
   */
  protected function getEditableConfigNames() {
    return [self::CONFIG];
  }

  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'queue_order_config_form';
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state) {
    $order_list = $this->config(self::CONFIG)->get('order');
    /** @var \Drupal\Core\Queue\QueueWorkerManagerInterface $queue */
    $queue = \Drupal::service('plugin.manager.queue_worker');
    $form[$this->getFormId()] = [
      '#type' => 'table',
      '#header' => [
        $this->t('Label'),
        $this->t('Machine name'),
        $this->t('Weight'),
        /*$this->t('Operations'),*/
      ],
      '#empty' => $this->t('There are no queues yet.'),
      '#tabledrag' => [
        [
          'action' => 'order',
          'relationship' => 'sibling',
          'group' => "{$this->getFormId()}-order-weight",
        ],
      ],
    ];
    foreach ($queue->getDefinitions() as $name => $definition) {
      $weight = empty($order_list[$name]) ? 0 : $order_list[$name];
      $form[$this->getFormId()][$name]['#attributes']['class'][] = 'draggable';
      $form[$this->getFormId()][$name]['#weight'] = $weight;
      $form[$this->getFormId()][$name]['label'] = [
        '#plain_text' => $definition['title'],
      ];
      $form[$this->getFormId()][$name]['id'] = [
        '#plain_text' => $name,
      ];
      $form[$this->getFormId()][$name]['weight'] = [
        '#type' => 'weight',
        '#title' => t('Weight for @title', ['@title' => $definition['title']]),
        '#title_display' => 'invisible',
        '#default_value' => $weight,
        '#attributes' => ['class' => ["{$this->getFormId()}-order-weight"]],
      ];
    }
    return parent::buildForm($form, $form_state);
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    $new_configs = [];
    $form_values = $form_state->getValue($this->getFormId());
    foreach ($form_values as $name => $array) {
      $new_configs[$name] = intval($array['weight']);
    }
    $this->config(self::CONFIG)
      ->set('order', $new_configs)
      ->save();
    parent::submitForm($form, $form_state);
  }

}
